@@ -24,9 +24,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #ifdef WIN32
 #define WIN32_LEAN_AND_MEAN
@@ -49,18 +47,11 @@
 #endif
 #include <errno.h>
 #include <string.h>
-#include <err.h>
 #include <assert.h>
 
-#ifdef USE_LOG
-#include "log.h"
-#else
-#define LOG_DBG(x)
-#define log_error(x)	perror(x)
-#endif
-
 #include "event.h"
 #include "event-internal.h"
+#include "log.h"
 
 #ifdef HAVE_SELECT
 extern const struct eventop selectops;
@@ -117,7 +108,7 @@ struct event_base *current_base = NULL;
 
 /* Handle signals - This is a deprecated interface */
 int (*event_sigcb)(void);	/* Signal callback when gotsig is set */
-int event_gotsig;		/* Set in signal handler */
+volatile int event_gotsig;	/* Set in signal handler */
 
 /* Prototypes */
 static void	event_queue_insert(struct event_base *, struct event *, int);
@@ -155,17 +146,12 @@ event_init(void)
 	int i;
 
 	if ((current_base = calloc(1, sizeof(struct event_base))) == NULL)
-		err(1, "%s: calloc");
+		event_err(1, "%s: calloc");
 
 	event_sigcb = NULL;
 	event_gotsig = 0;
 	gettimeofday(&current_base->event_tv, NULL);
 	
-#if defined(USE_LOG) && defined(USE_DEBUG)
-	log_to(stderr);
-	log_debug_cmd(LOG_MISC, 80);
-#endif
-
 	RB_INIT(&current_base->timetree);
 	TAILQ_INIT(&current_base->eventqueue);
 	TAILQ_INIT(&signalqueue);
@@ -178,11 +164,11 @@ event_init(void)
 	}
 
 	if (current_base->evbase == NULL)
-		errx(1, "%s: no event mechanism available", __func__);
+		event_errx(1, "%s: no event mechanism available", __func__);
 
 	if (getenv("EVENT_SHOW_METHOD")) 
-		fprintf(stderr, "libevent using: %s\n",
-		    current_base->evsel->name); 
+		event_msgx("libevent using: %s\n",
+			   current_base->evsel->name);
 
 	/* allocate a single active event queue */
 	event_base_priority_init(current_base, 1);
@@ -216,12 +202,12 @@ event_base_priority_init(struct event_base *base, int npriorities)
 	base->activequeues = (struct event_list **)calloc(base->nactivequeues,
 	    npriorities * sizeof(struct event_list *));
 	if (base->activequeues == NULL)
-		err(1, "%s: calloc", __func__);
+		event_err(1, "%s: calloc", __func__);
 
 	for (i = 0; i < base->nactivequeues; ++i) {
 		base->activequeues[i] = malloc(sizeof(struct event_list));
 		if (base->activequeues[i] == NULL)
-			err(1, "%s: malloc", __func__);
+			event_err(1, "%s: malloc", __func__);
 		TAILQ_INIT(base->activequeues[i]);
 	}
 
@@ -304,6 +290,13 @@ event_loopexit(struct timeval *tv)
 		    current_base, tv));
 }
 
+int
+event_base_loopexit(struct event_base *event_base, struct timeval *tv)
+{
+	return (event_once(-1, EV_TIMEOUT, event_loopexit_cb,
+		    event_base, tv));
+}
+
 /* not thread safe */
 
 int
@@ -348,10 +341,8 @@ event_base_loop(struct event_base *base, int flags)
 		gettimeofday(&tv, NULL);
 		if (timercmp(&tv, &base->event_tv, <)) {
 			struct timeval off;
-			LOG_DBG((LOG_MISC, 10,
-				    "%s: time is running backwards, corrected",
+			event_debug(("%s: time is running backwards, corrected",
 				    __func__));
-
 			timersub(&base->event_tv, &tv, &off);
 			timeout_correct(base, &off);
 		}
@@ -363,8 +354,10 @@ event_base_loop(struct event_base *base, int flags)
 			timerclear(&tv);
 		
 		/* If we have no events, we just exit */
-		if (!event_haveevents(base))
+		if (!event_haveevents(base)) {
+			event_debug(("%s: no events registered.", __func__));
 			return (1);
+		}
 
 		res = evsel->dispatch(base, evbase, &tv);
 
@@ -384,6 +377,7 @@ event_base_loop(struct event_base *base, int flags)
 			return (-1);
 	}
 
+	event_debug(("%s: asked to terminate loop.", __func__));
 	return (0);
 }
 
@@ -423,15 +417,15 @@ event_once(int fd, short events,
 	if ((eonce = calloc(1, sizeof(struct event_once))) == NULL)
 		return (-1);
 
+	eonce->cb = callback;
+	eonce->arg = arg;
+
 	if (events == EV_TIMEOUT) {
 		if (tv == NULL) {
 			timerclear(&etv);
 			tv = &etv;
 		}
 
-		eonce->cb = callback;
-		eonce->arg = arg;
-
 		evtimer_set(&eonce->ev, event_once_cb, eonce);
 	} else if (events & (EV_READ|EV_WRITE)) {
 		events &= EV_READ|EV_WRITE;
@@ -439,6 +433,7 @@ event_once(int fd, short events,
 		event_set(&eonce->ev, fd, events, event_once_cb, eonce);
 	} else {
 		/* Bad event combination */
+		free(eonce);
 		return (-1);
 	}
 
@@ -456,12 +451,7 @@ event_set(struct event *ev, int fd, short events,
 
 	ev->ev_callback = callback;
 	ev->ev_arg = arg;
-#ifdef WIN32
-	ev->ev_fd = (HANDLE)fd;
-	ev->overlap.hEvent = ev;
-#else
 	ev->ev_fd = fd;
-#endif
 	ev->ev_events = events;
 	ev->ev_flags = EVLIST_INIT;
 	ev->ev_ncalls = 0;
@@ -479,7 +469,7 @@ event_base_set(struct event_base *base, struct event *ev)
 		return (-1);
 
 	ev->ev_base = base;
-	ev->ev_pri = current_base->nactivequeues/2;
+	ev->ev_pri = base->nactivequeues/2;
 
 	return (0);
 }
@@ -536,7 +526,7 @@ event_add(struct event *ev, struct timeval *tv)
 	const struct eventop *evsel = base->evsel;
 	void *evbase = base->evbase;
 
-	LOG_DBG((LOG_MISC, 55,
+	event_debug((
 		 "event_add: event: %p, %s%s%scall %p",
 		 ev,
 		 ev->ev_events & EV_READ ? "EV_READ " : " ",
@@ -571,7 +561,7 @@ event_add(struct event *ev, struct timeval *tv)
 		gettimeofday(&now, NULL);
 		timeradd(&now, tv, &ev->ev_timeout);
 
-		LOG_DBG((LOG_MISC, 55,
+		event_debug((
 			 "event_add: timeout in %d seconds, call %p",
 			 tv->tv_sec, ev->ev_callback));
 
@@ -600,7 +590,7 @@ event_del(struct event *ev)
 	const struct eventop *evsel;
 	void *evbase;
 
-	LOG_DBG((LOG_MISC, 80, "event_del: %p, callback %p",
+	event_debug(("event_del: %p, callback %p",
 		 ev, ev->ev_callback));
 
 	/* An event without a base has not been added */
@@ -677,7 +667,7 @@ timeout_next(struct event_base *base, struct timeval *tv)
 	assert(tv->tv_sec >= 0);
 	assert(tv->tv_usec >= 0);
 
-	LOG_DBG((LOG_MISC, 60, "timeout_next: in %d seconds", tv->tv_sec));
+	event_debug(("timeout_next: in %d seconds", tv->tv_sec));
 	return (0);
 }
 
@@ -712,7 +702,7 @@ timeout_process(struct event_base *base)
 		/* delete this event from the I/O queues */
 		event_del(ev);
 
-		LOG_DBG((LOG_MISC, 60, "timeout_process: call %p",
+		event_debug(("timeout_process: call %p",
 			 ev->ev_callback));
 		event_active(ev, EV_TIMEOUT, 1);
 	}
@@ -724,8 +714,8 @@ event_queue_remove(struct event_base *base, struct event *ev, int queue)
 	int docount = 1;
 
 	if (!(ev->ev_flags & queue))
-		errx(1, "%s: %p(fd %d) not on queue %x", __func__,
-		    ev, ev->ev_fd, queue);
+		event_errx(1, "%s: %p(fd %d) not on queue %x", __func__,
+			   ev, ev->ev_fd, queue);
 
 	if (ev->ev_flags & EVLIST_INTERNAL)
 		docount = 0;
@@ -751,7 +741,7 @@ event_queue_remove(struct event_base *base, struct event *ev, int queue)
 		TAILQ_REMOVE(&base->eventqueue, ev, ev_next);
 		break;
 	default:
-		errx(1, "%s: unknown queue %x", __func__, queue);
+		event_errx(1, "%s: unknown queue %x", __func__, queue);
 	}
 }
 
@@ -765,8 +755,8 @@ event_queue_insert(struct event_base *base, struct event *ev, int queue)
 		if (queue & EVLIST_ACTIVE)
 			return;
 
-		errx(1, "%s: %p(fd %d) already on queue %x", __func__,
-		    ev, ev->ev_fd, queue);
+		event_errx(1, "%s: %p(fd %d) already on queue %x", __func__,
+			   ev, ev->ev_fd, queue);
 	}
 
 	if (ev->ev_flags & EVLIST_INTERNAL)
@@ -795,6 +785,25 @@ event_queue_insert(struct event_base *base, struct event *ev, int queue)
 		TAILQ_INSERT_TAIL(&base->eventqueue, ev, ev_next);
 		break;
 	default:
-		errx(1, "%s: unknown queue %x", __func__, queue);
+		event_errx(1, "%s: unknown queue %x", __func__, queue);
 	}
 }
+
+/* Functions for debugging */
+
+const char *
+event_get_version(void)
+{
+	return (VERSION);
+}
+
+/* 
+ * No thread-safe interface needed - the information should be the same
+ * for all threads.
+ */
+
+const char *
+event_get_method(void)
+{
+	return (current_base->evsel->name);
+}
